#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, sys
import math
from gimpfu import *
import xml.etree.ElementTree as ET

# Creates a 4-point wind rose on a new layer based on size the layer or selection.
# (c) MareroQ

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release.

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

def newline(x1, y1, x2, y2):
    return [x1,y1]*3+[x2,y2]*3

def plugin_roses_compass(image, layer, sel, fill, color_1, color_2,stroke, strokecolor, strokewidth, path_v):

	#gimp.context_push()
	#image.undo_group_start()
	alpha=pdb.gimp_drawable_has_alpha(layer)
	if (alpha==False):
		pdb.gimp_layer_add_alpha (layer)
		
	if sel == False:
		pdb.gimp_selection_all(image)
	selection = pdb.gimp_selection_bounds(image)
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];

	if sel:
		temp = pdb.gimp_selection_save(image)
		W1 = 2*(x1+(x2 - x1)/2)
		H1 = 2*(y1+(y2 - y1)/2)
		j = x1
		k = y1
		if x2-x1>=y2-y1:
			m = (x1+x2-y2+y1)/2
			v = y1
			W = (y2 - y1)
			H = (y2 - y1)
		else:
			m = x1
			v = (y1+y2-x2+x1)/2
			W = x2 - x1
			H = x2 - x1
	else:
		W1 = pdb.gimp_image_width(image)  #szer
		H1 = pdb.gimp_image_height(image) #wys
		k = 0
		j = 0
		if W1>H1:
			m =(W1-H1)/2
			v = 0
			W = H1
			H = H1

		else:
			m = 0
			v = (H1-W1)/2
			W = W1
			H = W1
	temp = pdb.gimp_selection_save(image)

	inWidth = W
	inHeight = H

	theImage = image
	if (image.width < inWidth) or (image.height < inHeight):        
		pdb.gimp_image_resize(theImage, max(inWidth, image.width), max(inHeight, image.height), 0, 0)
	baseLayer = pdb.gimp_layer_new(theImage, inWidth, inHeight, RGBA_IMAGE,  "Roses_compass", 100, LAYER_MODE_NORMAL_LEGACY)
	theImage.add_layer(baseLayer)
     
	baseLayer.fill(FILL_TRANSPARENT)	       #GIMP 2.9

	sel = pdb.gimp_image_get_selection(theImage)
#shape code   
	pdb.gimp_layer_resize_to_image_size(baseLayer)
	pdb.gimp_selection_none(image)
	pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m,v,  3*W/8+m,3*W/8+v, W/2+m, W/2+v, W/2+m,v))
	pdb.gimp_image_select_polygon(image, 0, 8, (m, W/2+v, 3*W/8+m, 5*W/8+v, W/2+m, W/2+v, m, W/2+v))
	pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, W/2+m, W+v, 5*W/8 +m, 5*W/8 +v,W/2+m, W/2+v))
	pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, W+m, W/2+v, 5*W/8 +m, 3*W/8 +v, W/2+m, W/2+v))
	pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	active_vectors.visible = True
	active_vectors.name = active_vectors.name.replace("Selection", "Roses_L")
	active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Róża wiatrów L")
	if fill==1:
		pdb.gimp_image_select_item(image, 2, active_vectors)
		pdb.gimp_context_set_foreground(color_1)
		pdb.gimp_edit_fill(baseLayer,FILL_FOREGROUND)
	pdb.gimp_selection_none(image)
	
	pdb.gimp_selection_none(image)
	pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m,v, W/2+m,W/2 +v, 5*W/8+m,3*W/8+v, W/2+m,v))
	pdb.gimp_image_select_polygon(image, 0, 8, (3*W/8+m,3*W/8+v, m, W/2+v, W/2+m, W/2+v, 3*W/8+m,3*W/8+v))
	pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, 3*W/8+m,5*W/8+v, W/2+m, W+v, W/2+m, W/2+v,))
	pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, 5*W/8 +m, 5*W/8 +v, W+m, W/2+v,  W/2+m, W/2+v,))
	pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	active_vectors.visible = True
	active_vectors.name = active_vectors.name.replace("Selection", "Roses_R")
	if fill==1:	
		pdb.gimp_context_set_background(color_2)
		pdb.gimp_edit_fill(baseLayer,FILL_BACKGROUND)	      
		pdb.gimp_selection_none(image)

	pdb.gimp_selection_none(image)
	
	if stroke==1:
		pdb.gimp_selection_none(image)
		W = pdb.gimp_image_width(image)
		H = pdb.gimp_image_height(image)
		stroke_b = gimp.Layer(image, "Stroke brush", W, H, RGBA_IMAGE, 100, LAYER_MODE_NORMAL_LEGACY)
		image.add_layer(stroke_b, -1)
		pdb.gimp_context_set_stroke_method(0)
		pdb.gimp_context_set_brush('1. Pixel')
		pdb.gimp_context_set_foreground(strokecolor)
		pdb.gimp_context_set_line_width(strokewidth)
		pdb.gimp_drawable_edit_stroke_item(stroke_b, pdb.gimp_image_get_vectors_by_name(image, 'Roses_R'))
		pdb.gimp_drawable_edit_stroke_item(stroke_b, pdb.gimp_image_get_vectors_by_name(image, 'Roses_L'))
	if path_v==0:		
		pdb.gimp_item_set_visible(pdb.gimp_image_get_vectors_by_name(image, 'Roses_R'), 0)		
		pdb.gimp_item_set_visible(pdb.gimp_image_get_vectors_by_name(image, 'Roses_L'), 0)	

	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
        "plugin_roses_compass",
        "Creates a 4-point wind rose on a new layer\nbased on size the layer or selection. Ver.1",
        "Creates a 4-point wind rose on a new layer based on size the layer or selection.",
        "MrQ",
        "GPL",
        "2023",
        "<Image>/Filters/Path/Roses Compass 4 point...",
        "RGB*",
        [
		#(PF_IMAGE, "image", "Input image", None),
		#(PF_VECTORS, "drawable", "Input drawable", None),
		(PF_BOOL, "sel", ("In selection? "), False),
		(PF_OPTION, "fill", "FILL (path to selection)?:", 1, ["Don't fill",  "YES"]),
		(PF_COLOR,   "color_sl", "Color 1: ", (240, 240, 10)),
		(PF_COLOR,   "color_sl", "Color 2: ", (20, 110, 250)),
		(PF_OPTION, "stroke", "Stroke lines?:", 1, ["Don't stroke", "YES"]),		
		(PF_COLOR, "strokecolor",  "Stroke Path Color:",  (0,0,0)),
		(PF_SPINNER, "strokewidth", "Line Thickness (pix):", 2, (1, 10, 1)),
		(PF_TOGGLE , 'path_v', "Path visible?", 0,["No","YES"])
		],
		[],
		plugin_roses_compass)

main()